use std::path::PathBuf;

use gtk::prelude::*;
use log::debug;
use relm4::factory::FactoryVecDeque;
use relm4::{
    gtk, Component, ComponentController, ComponentParts, ComponentSender, SimpleComponent,
};
use relm4::{prelude::*, RelmListBoxExt};
use walkdir::WalkDir;

use super::sheet_model::{OnQueryUpdate, SheetModel, SheetModelType};

pub struct SheetListingModel {
    sheets: FactoryVecDeque<SheetModel>,
}

#[derive(Debug)]
pub enum SheetListingInput {
    Query(String),
    ListBoxRowClicked(i32),
}

pub struct SheetModelSelected {
    sheet_model_type: SheetModelType,
}

#[relm4::component(pub)]
impl SimpleComponent for SheetListingModel {
    type Init = Vec<SheetModelType>;
    type Input = SheetListingInput;
    type Output = ();

    view! {
        #[root]
        gtk::Box {
            // set_orientation: gtk::Orientation::Vertical,

            model.sheets.widget() -> &relm4::gtk::ListBox {
                set_hexpand: true,
                // set_orientation: gtk::Orientation::Vertical,
                // set_spacing: 5,
                connect_row_activated[sender] => move |list_box, row| {
                    let index = list_box.index_of_child(row).unwrap();
                    sender.input(SheetListingInput::ListBoxRowClicked(index));
                },
            },
        }
    }

    fn init(
        init: Self::Init,
        root: &Self::Root,
        sender: ComponentSender<Self>,
    ) -> ComponentParts<Self> {
        let mut sheets = FactoryVecDeque::new(gtk::ListBox::default(), sender.input_sender());
        for sheet_model_type in init {
            sheets.guard().push_back(sheet_model_type);
        }

        let model = SheetListingModel { sheets };
        let widgets = view_output!();
        ComponentParts { model, widgets }
    }

    fn update(&mut self, message: Self::Input, _sender: ComponentSender<Self>) {
        match message {
            SheetListingInput::Query(query) => {
                self.sheets.broadcast(OnQueryUpdate { query });
            }
            SheetListingInput::ListBoxRowClicked(index) => {
                let x = self.sheets.get(index as usize).unwrap();
                debug!("clicked: {}!!!!!", x.label);
            }
        }
    }
}
